#ifndef TRANSFERMANAGER_H
#define TRANSFERMANAGER_H

#include "transferitem.h"
#include <QObject>

class TransferManager : public QObject
{
    Q_OBJECT

public:
    explicit TransferManager(QObject *parent = 0);
    ~TransferManager();
    inline QNetworkAccessManager* networkAccessManager() const { return m_nam; }
    inline void setNetworkAccessManager(QNetworkAccessManager *manager) { m_nam = manager; }
    Q_INVOKABLE void restoreStoredDownloads();
    Q_INVOKABLE int concurrentTransfers() const;
    Q_INVOKABLE inline int transferCount() const { return this->transferList()->size(); }
    inline QList< QSharedPointer<TransferItem> >* transferList() const { return m_transfers; }
    Q_INVOKABLE void addDownloadTransfer(TrackItem *track, bool notify = true);
    void addDownloadTransfer(QSharedPointer<TrackItem> track, bool notify = true);
    Q_INVOKABLE void addDownloadTransfer(PlaylistItem *playlist, QList<TrackItem*> tracks, bool notify = true);
    void addDownloadTransfer(QSharedPointer<PlaylistItem> playlist, QList< QSharedPointer<TrackItem> >tracks, bool notify = true);
    Q_INVOKABLE void addDownloadTransfers(QList<TrackItem*> tracks);
    void addDownloadTransfers(QList< QSharedPointer<TrackItem> >tracks);

    static TransferManager* instance();

public slots:
    void onMaximumConcurrentTransfersChanged(int oldMaximum, int newMaximum);

private:
    QString generateTransferId(QString seedFileName) const;
    QSharedPointer<TransferItem> getNextTransfer() const;
    QSharedPointer<TransferItem> getTransfer(TransferItem *transfer) const;
    bool removeTransfer(QSharedPointer<TransferItem> transfer);

private slots:
    void onTransferStatusChanged(Transfers::TransferStatus status);
    void onTransferCompleted(QSharedPointer<TransferItem> transfer);
    void onTransferCancelled(QSharedPointer<TransferItem> transfer);
    
signals:
    void alert(const QString &message);
    void transferRemoved(int row);

private:
    QNetworkAccessManager *m_nam;
    QList< QSharedPointer<TransferItem> > *m_transfers;
};

#endif // TRANSFERMANAGER_H
